import sys
from enum import Enum
from typing import Union

import dotenv
from dotenv import load_dotenv
import os
from loguru import logger


class LogLevel(Enum):
    NONE = "NONE"
    TRACE = "TRACE"
    DEBUG = "DEBUG"
    WARNING = "WARNING"
    INFO = "INFO"
    ERROR = "ERROR"


LOG_LEVELS = [loader.value for loader in LogLevel]


def configure_logger(log_level: Union[str, LogLevel]):
    level = log_level.value if isinstance(log_level, LogLevel) else log_level
    # yes, it is duplicate but it is nice to avoid cross-module dependencies here
    if level.upper() != LogLevel.NONE.value:
        logger.add(sys.stdout, level=level.upper())
    return logger


def load_environment_keys(debug: bool = True, usecwd: bool = False):
    e = dotenv.find_dotenv(usecwd=usecwd)
    print(e)
    if debug:
        print(f"environment found at {e}")
    has_env: bool = load_dotenv(e, verbose=True, override=True)
    if not has_env:
        print("Did not found environment file, using system OpenAI key (if exists)")
    openai_key = os.getenv('OPENAI_API_KEY')
    return openai_key


import json

import click
from pycomfort.files import replace_in_file, replace_from_dict_in_file
from pathlib import Path
from typing import Union, Optional


@click.command("replace")
@click.option('--file', type=click.Path(exists=True), help="rename file with substitution")
@click.option('--what', help="substitute --from")
@click.option('--to', help="substitute --to")
@click.option('--output', help="optional output, will rewrite --file is not output provided")
def replace(file: str, what: str, to: str, output: Optional[str]):
    print(f"replacing {what} to {to} in {file}")
    where = None if output is None else Path(output)
    return replace_in_file(Path(file), what, to, where)


@click.command("replace_with_dictionary")
@click.option('--file', type=click.Path(exists=True), help="rename file with substitution")
@click.option('--dictionary', type=click.Path(exists=True), help="dictionary to load from")
@click.option('--output', help="optional output, will rewrite --file is not output provided")
@click.option('--verbose', type=click.BOOL, help="if we should output more to console")
def replace_dict(file: str, dictionary: str, output: Optional[str], verbose: bool = False):
    print(f"replacing from {dict} in {file}")
    # reading the data from the file
    with Path(dictionary).open("r+") as f:
        data = f.read()
    js: dict = json.loads(data)
    print(f"substitutions dictionary is:")
    for k, v in js.items():
        print(f"REPLACE {k} WITH {v}\n")
    where = None if output is None else Path(output)
    return replace_from_dict_in_file(Path(file), js, where, verbose)


if __name__ == '__main__':
    replace()